@@ -61,23 +61,26 @@ import java.util.HashMap;
 import java.util.Map;
 import java.util.Set;
 
+import org.apache.commons.io.IOUtils;
 import org.apache.tika.config.TikaConfig;
 import org.apache.tika.exception.TikaException;
 import org.apache.tika.extractor.DocumentSelector;
-import org.apache.tika.io.IOUtils;
 import org.apache.tika.io.TikaInputStream;
 import org.apache.tika.metadata.Metadata;
 import org.apache.tika.metadata.serialization.JsonMetadataList;
 import org.apache.tika.mime.MediaType;
 import org.apache.tika.parser.AbstractParser;
 import org.apache.tika.parser.AutoDetectParser;
+import org.apache.tika.parser.DigestingParser;
 import org.apache.tika.parser.ParseContext;
 import org.apache.tika.parser.Parser;
 import org.apache.tika.parser.RecursiveParserWrapper;
 import org.apache.tika.parser.html.BoilerpipeContentHandler;
+import org.apache.tika.parser.utils.CommonsDigester;
 import org.apache.tika.sax.BasicContentHandlerFactory;
 import org.apache.tika.sax.BodyContentHandler;
 import org.apache.tika.sax.ContentHandlerDecorator;
+import org.apache.tika.sax.RecursiveParserWrapperHandler;
 import org.apache.tika.sax.TeeContentHandler;
 import org.apache.tika.sax.XHTMLContentHandler;
 import org.xml.sax.Attributes;
@@ -85,6 +88,8 @@ import org.xml.sax.ContentHandler;
 import org.xml.sax.SAXException;
 import org.xml.sax.helpers.AttributesImpl;
 
+import static java.nio.charset.StandardCharsets.UTF_8;
+
 /**
  * Simple Swing GUI for Apache Tika. You can drag and drop files on top
  * of the window to have them parsed.
@@ -92,6 +97,9 @@ import org.xml.sax.helpers.AttributesImpl;
 public class TikaGUI extends JFrame
         implements ActionListener, HyperlinkListener {
 
+    //maximum length to allow for mark for reparse to get JSON
+    private static final int MAX_MARK = 20*1024*1024;//20MB
+
     /**
      * Serial version UID.
      */
@@ -115,13 +123,16 @@ public class TikaGUI extends JFrame
         final TikaConfig finalConfig = config;
         SwingUtilities.invokeLater(new Runnable() {
             public void run() {
-                new TikaGUI(new AutoDetectParser(finalConfig)).setVisible(true);
+                new TikaGUI(new DigestingParser(
+                        new AutoDetectParser(finalConfig),
+                        new CommonsDigester(MAX_MARK,
+                                CommonsDigester.DigestAlgorithm.MD5,
+                                CommonsDigester.DigestAlgorithm.SHA256)
+                        )).setVisible(true);
             }
         });
     }
 
-    //maximum length to allow for mark for reparse to get JSON
-    private final int MAX_MARK = 20*1024*1024;//20MB
     /**
      * Parsing context.
      */
@@ -294,11 +305,8 @@ public class TikaGUI extends JFrame
     public void openFile(File file) {
         try {
             Metadata metadata = new Metadata();
-            TikaInputStream stream = TikaInputStream.get(file, metadata);
-            try {
+            try (TikaInputStream stream = TikaInputStream.get(file, metadata)) {
                 handleStream(stream, metadata);
-            } finally {
-                stream.close();
             }
         } catch (Throwable t) {
             handleError(file.getPath(), t);
@@ -308,11 +316,8 @@ public class TikaGUI extends JFrame
     public void openURL(URL url) {
         try {
             Metadata metadata = new Metadata();
-            TikaInputStream stream = TikaInputStream.get(url, metadata);
-            try {
+            try (TikaInputStream stream = TikaInputStream.get(url, metadata)) {
                 handleStream(stream, metadata);
-            } finally {
-                stream.close();
             }
         } catch (Throwable t) {
             handleError(url.toString(), t);
@@ -334,20 +339,33 @@ public class TikaGUI extends JFrame
                 getXmlContentHandler(xmlBuffer));
 
         context.set(DocumentSelector.class, new ImageDocumentSelector());
+
+        input = TikaInputStream.get(new ProgressMonitorInputStream(
+                this, "Parsing stream", input));
+
         if (input.markSupported()) {
-            input.mark(MAX_MARK);
+            int mark = -1;
+            if (input instanceof TikaInputStream) {
+                if (((TikaInputStream)input).hasFile()) {
+                    mark = (int)((TikaInputStream)input).getLength();
+                }
+            }
+            if (mark == -1) {
+                mark = MAX_MARK;
+            }
+            input.mark(mark);
         }
-        input = new ProgressMonitorInputStream(
-                this, "Parsing stream", input);
         parser.parse(input, handler, md, context);
 
         String[] names = md.names();
         Arrays.sort(names);
         for (String name : names) {
-            metadataBuffer.append(name);
-            metadataBuffer.append(": ");
-            metadataBuffer.append(md.get(name));
-            metadataBuffer.append("\n");
+            for (String val : md.getValues(name)) {
+                metadataBuffer.append(name);
+                metadataBuffer.append(": ");
+                metadataBuffer.append(val);
+                metadataBuffer.append("\n");
+            }
         }
 
         String name = md.get(Metadata.RESOURCE_NAME_KEY);
@@ -378,13 +396,16 @@ public class TikaGUI extends JFrame
             );
         }
         if (isReset) {
-            RecursiveParserWrapper wrapper = new RecursiveParserWrapper(parser,
-                    new BasicContentHandlerFactory(
-                            BasicContentHandlerFactory.HANDLER_TYPE.BODY, -1));
-            wrapper.parse(input, null, new Metadata(), new ParseContext());
+            RecursiveParserWrapperHandler recursiveParserWrapperHandler =
+                    new RecursiveParserWrapperHandler(
+                            new BasicContentHandlerFactory(
+                                    BasicContentHandlerFactory.HANDLER_TYPE.BODY, -1),
+                            -1);
+            RecursiveParserWrapper wrapper = new RecursiveParserWrapper(parser);
+            wrapper.parse(input, recursiveParserWrapperHandler, new Metadata(), new ParseContext());
             StringWriter jsonBuffer = new StringWriter();
             JsonMetadataList.setPrettyPrinting(true);
-            JsonMetadataList.toJson(wrapper.getMetadata(), jsonBuffer);
+            JsonMetadataList.toJson(recursiveParserWrapperHandler.getMetadataList(), jsonBuffer);
             setText(json, jsonBuffer.toString());
         }
         layout.show(cards, "metadata");
@@ -456,13 +477,9 @@ public class TikaGUI extends JFrame
         if (e.getEventType() == EventType.ACTIVATED) {
             try {
                 URL url = e.getURL();
-                InputStream stream = url.openStream();
-                try {
-                    StringWriter writer = new StringWriter();
-                    IOUtils.copy(stream, writer, IOUtils.UTF_8.name());
-
+                try (InputStream stream = url.openStream()) {
                     JEditorPane editor =
-                        new JEditorPane("text/plain", writer.toString());
+                        new JEditorPane("text/plain", IOUtils.toString(stream, UTF_8));
                     editor.setEditable(false);
                     editor.setBackground(Color.WHITE);
                     editor.setCaretPosition(0);
@@ -475,8 +492,6 @@ public class TikaGUI extends JFrame
                     dialog.add(new JScrollPane(editor));
                     dialog.pack();
                     dialog.setVisible(true);
-                } finally {
-                    stream.close();
                 }
             } catch (IOException exception) {
                 exception.printStackTrace();
