@@ -1,5 +1,3 @@
-package org.apache.tika.batch.builders;
-
 /*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
@@ -16,6 +14,7 @@ package org.apache.tika.batch.builders;
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
+package org.apache.tika.batch.builders;
 
 import java.util.Map;
 
@@ -27,7 +26,7 @@ import org.w3c.dom.Node;
  */
 public class InterrupterBuilder {
 
-    public Interrupter build(Node n, Map<String, String> commandlineArguments) {
-        return new Interrupter();
+    public Interrupter build(Node n, long pauseOnEarlyTermination, Map<String, String> commandlineArguments) {
+        return new Interrupter(pauseOnEarlyTermination);
     }
 }
