@@ -21,9 +21,11 @@ import org.apache.tika.batch.ParserFactory;
 import org.apache.tika.config.TikaConfig;
 import org.apache.tika.parser.Parser;
 
-public class MockParserFactory implements ParserFactory {
+public class MockParserFactory extends ParserFactory {
+
     @Override
     public Parser getParser(TikaConfig config) {
         return new MockParser();
     }
+
 }
