@@ -16,11 +16,14 @@
  */
 package org.apache.tika.bundle;
 
+import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
-import static org.junit.Assert.fail;
-import static org.ops4j.pax.exam.CoreOptions.*;
+import static org.ops4j.pax.exam.CoreOptions.bundle;
+import static org.ops4j.pax.exam.CoreOptions.junitBundles;
+import static org.ops4j.pax.exam.CoreOptions.mavenBundle;
+import static org.ops4j.pax.exam.CoreOptions.options;
 
 import java.io.ByteArrayInputStream;
 import java.io.File;
@@ -30,25 +33,29 @@ import java.io.InputStream;
 import java.io.StringWriter;
 import java.io.Writer;
 import java.net.URISyntaxException;
+import java.nio.file.Paths;
 import java.util.HashSet;
-import java.util.List;
 import java.util.Set;
+import java.util.jar.Attributes;
+import java.util.jar.JarInputStream;
+import java.util.jar.Manifest;
 
 import javax.inject.Inject;
 
 import org.apache.tika.Tika;
-import org.apache.tika.config.ServiceLoader;
-import org.apache.tika.config.TikaConfig;
 import org.apache.tika.detect.DefaultDetector;
 import org.apache.tika.detect.Detector;
 import org.apache.tika.fork.ForkParser;
+import org.apache.tika.io.TikaInputStream;
 import org.apache.tika.metadata.Metadata;
 import org.apache.tika.mime.MediaType;
+import org.apache.tika.parser.CompositeParser;
+import org.apache.tika.parser.DefaultParser;
 import org.apache.tika.parser.ParseContext;
 import org.apache.tika.parser.Parser;
 import org.apache.tika.parser.internal.Activator;
+import org.apache.tika.parser.ocr.TesseractOCRParser;
 import org.apache.tika.sax.BodyContentHandler;
-import org.junit.Ignore;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.ops4j.pax.exam.Configuration;
@@ -58,31 +65,35 @@ import org.ops4j.pax.exam.spi.reactors.ExamReactorStrategy;
 import org.ops4j.pax.exam.spi.reactors.PerMethod;
 import org.osgi.framework.Bundle;
 import org.osgi.framework.BundleContext;
+import org.osgi.framework.ServiceReference;
 import org.xml.sax.ContentHandler;
 
 @RunWith(PaxExam.class)
 @ExamReactorStrategy(PerMethod.class)
 public class BundleIT {
-
-    private final File TARGET = new File("target");
-
+	
+	private final File TARGET = new File("target");
+	
     @Inject
     private Parser defaultParser;
+
     @Inject
     private Detector contentTypeDetector;
+
     @Inject
     private BundleContext bc;
 
     @Configuration
-    public Option[] configuration() throws IOException, URISyntaxException {
-        File base = new File(TARGET, "test-bundles");
+    public Option[] configuration() throws IOException, URISyntaxException, ClassNotFoundException {
+    	File base = new File(TARGET, "test-bundles");
         return options(
-                junitBundles(),
-                bundle(new File(base, "tika-core.jar").toURI().toURL().toString()),
-                bundle(new File(base, "tika-bundle.jar").toURI().toURL().toString()));
+        		bundle(new File(base, "tika-core.jar").toURI().toURL().toString()),
+        		mavenBundle("org.ops4j.pax.logging", "pax-logging-api", "1.8.5"),
+        		mavenBundle("org.ops4j.pax.logging", "pax-logging-service", "1.8.5"),
+        		junitBundles(),
+        		bundle(new File(base, "tika-bundle.jar").toURI().toURL().toString()));
     }
 
-
     @Test
     public void testBundleLoaded() throws Exception {
         boolean hasCore = false, hasBundle = false;
@@ -100,22 +111,42 @@ public class BundleIT {
         assertTrue("Bundle bundle not found", hasBundle);
     }
 
+    @Test
+    public void testManifestNoJUnit() throws Exception {
+        File TARGET = new File("target");
+        File base = new File(TARGET, "test-bundles");
+        File tikaBundle = new File(base, "tika-bundle.jar");
+
+        JarInputStream jarIs = new JarInputStream(new FileInputStream(tikaBundle));
+        Manifest mf = jarIs.getManifest();
+
+        Attributes main = mf.getMainAttributes();
+
+        String importPackage = main.getValue("Import-Package");
+
+        boolean containsJunit = importPackage.contains("junit");
+
+        assertFalse("The bundle should not import junit", containsJunit);
+    }
 
     @Test
     public void testBundleDetection() throws Exception {
-        Tika tika = new Tika();
+        Metadata metadataTXT = new Metadata();
+        metadataTXT.set(Metadata.RESOURCE_NAME_KEY, "test.txt");
+
+        Metadata metadataPDF = new Metadata();
+        metadataPDF.set(Metadata.RESOURCE_NAME_KEY, "test.pdf");
 
         // Simple type detection
-        assertEquals("text/plain", tika.detect("test.txt"));
-        assertEquals("application/pdf", tika.detect("test.pdf"));
+        assertEquals(MediaType.TEXT_PLAIN, contentTypeDetector.detect(null, metadataTXT));
+        assertEquals(MediaType.application("pdf"), contentTypeDetector.detect(null, metadataPDF));
     }
 
-
     @Test
     public void testForkParser() throws Exception {
         ForkParser parser = new ForkParser(Activator.class.getClassLoader(), defaultParser);
         String data = "<!DOCTYPE html>\n<html><body><p>test <span>content</span></p></body></html>";
-        InputStream stream = new ByteArrayInputStream(data.getBytes("UTF-8"));
+        InputStream stream = new ByteArrayInputStream(data.getBytes(UTF_8));
         Writer writer = new StringWriter();
         ContentHandler contentHandler = new BodyContentHandler(writer);
         Metadata metadata = new Metadata();
@@ -130,8 +161,6 @@ public class BundleIT {
         assertEquals("test content", content.trim());
     }
 
-
-    @Ignore // TODO Fix this test
     @Test
     public void testBundleSimpleText() throws Exception {
         Tika tika = new Tika();
@@ -141,46 +170,83 @@ public class BundleIT {
         assertTrue(xml.contains("tika-bundle"));
     }
 
-
-    @Ignore // TODO Fix this test
     @Test
     public void testBundleDetectors() throws Exception {
-        // Get the raw detectors list
-        // TODO Why is this not finding the detector service resource files?
-        TestingServiceLoader loader = new TestingServiceLoader();
-        List<String> rawDetectors = loader.identifyStaticServiceProviders(Detector.class);
-
-        // Check we did get a few, just in case...
-        assertNotNull(rawDetectors);
-        assertTrue("Should have several Detector names, found " + rawDetectors.size(),
-                rawDetectors.size() > 3);
+        //For some reason, the detector created by OSGi has a flat
+        //list of detectors, whereas the detector created by the traditional
+        //service loading method has children: DefaultDetector, MimeTypes.
+        //We have to flatten the service loaded DefaultDetector to get equivalence.
+        //Detection behavior should all be the same.
 
         // Get the classes found within OSGi
-        DefaultDetector detector = new DefaultDetector();
-        Set<String> osgiDetectors = new HashSet<String>();
-        for (Detector d : detector.getDetectors()) {
+        ServiceReference<Detector> detectorRef = bc.getServiceReference(Detector.class);
+        DefaultDetector detectorService = (DefaultDetector) bc.getService(detectorRef);
+
+        Set<String> osgiDetectors = new HashSet<>();
+        for (Detector d : detectorService.getDetectors()) {
             osgiDetectors.add(d.getClass().getName());
         }
 
-        // Check that OSGi didn't miss any
-        for (String detectorName : rawDetectors) {
-            if (!osgiDetectors.contains(detectorName)) {
-                fail("Detector " + detectorName
-                        + " not found within OSGi Detector list: " + osgiDetectors);
+        // Check we did get a few, just in case...
+        assertTrue("Should have several Detector names, found " + osgiDetectors.size(),
+                osgiDetectors.size() > 3);
+
+        // Get the raw detectors list from the traditional service loading mechanism
+        DefaultDetector detector = new DefaultDetector();
+        Set<String> rawDetectors = new HashSet<String>();
+        for (Detector d : detector.getDetectors()) {
+            if (d instanceof DefaultDetector) {
+                for (Detector dChild : ((DefaultDetector) d).getDetectors()) {
+                    rawDetectors.add(dChild.getClass().getName());
+                }
+            } else {
+                rawDetectors.add(d.getClass().getName());
             }
         }
+        assertEquals(osgiDetectors, rawDetectors);
     }
 
-
     @Test
     public void testBundleParsers() throws Exception {
-        TikaConfig tika = new TikaConfig();
+        // Get the classes found within OSGi
+        ServiceReference<Parser> parserRef = bc.getServiceReference(Parser.class);
+        DefaultParser parserService = (DefaultParser) bc.getService(parserRef);
 
-        // TODO Implement as with Detectors
+        Set<String> osgiParsers = new HashSet<>();
+        for (Parser p : parserService.getAllComponentParsers()) {
+            osgiParsers.add(p.getClass().getName());
+        }
+
+        // Check we did get a few, just in case...
+        assertTrue("Should have lots Parser names, found " + osgiParsers.size(),
+                osgiParsers.size() > 15);
+
+        // Get the raw parsers list from the traditional service loading mechanism
+        CompositeParser parser = (CompositeParser) defaultParser;
+        Set<String> rawParsers = new HashSet<>();
+        for (Parser p : parser.getAllComponentParsers()) {
+            if (p instanceof DefaultParser) {
+                for (Parser pChild : ((DefaultParser) p).getAllComponentParsers()) {
+                    rawParsers.add(pChild.getClass().getName());
+                }
+            } else {
+                rawParsers.add(p.getClass().getName());
+            }
+        }
+        assertEquals(rawParsers, osgiParsers);
     }
 
+    @Test
+    public void testTesseractParser() throws Exception {
+        ContentHandler handler = new BodyContentHandler();
+        ParseContext context = new ParseContext();
+        Parser tesseractParser = new TesseractOCRParser();
+        try (InputStream stream = new FileInputStream("src/test/resources/testOCR.jpg")) {
+            tesseractParser.parse(stream, handler, new Metadata(), context);
+        }
+
+    }
 
-    @Ignore // TODO Fix this test
     @Test
     public void testTikaBundle() throws Exception {
         Tika tika = new Tika();
@@ -192,12 +258,8 @@ public class BundleIT {
         ParseContext context = new ParseContext();
         context.set(Parser.class, parser);
 
-        InputStream stream
-                = new FileInputStream("src/test/resources/test-documents.zip");
-        try {
+        try (InputStream stream = TikaInputStream.get(Paths.get("src/test/resources/test-documents.zip"))) {
             parser.parse(stream, handler, new Metadata(), context);
-        } finally {
-            stream.close();
         }
 
         String content = handler.toString();
@@ -220,19 +282,4 @@ public class BundleIT {
         assertTrue(content.contains("testXML.xml"));
         assertTrue(content.contains("Rida Benjelloun"));
     }
-
-    /**
-     * Alternate ServiceLoader which works outside of OSGi, so we can compare between the two environments
-     */
-    private static class TestingServiceLoader extends ServiceLoader {
-
-        private TestingServiceLoader() {
-            super();
-        }
-
-
-        public <T> List<String> identifyStaticServiceProviders(Class<T> iface) {
-            return super.identifyStaticServiceProviders(iface);
-        }
-    }
 }
