@@ -39,6 +39,8 @@ import org.apache.tika.mime.MediaType;
 import org.apache.tika.parser.ParseContext;
 import org.apache.tika.parser.external.ExternalParser;
 
+import static java.nio.charset.StandardCharsets.UTF_8;
+
 /**
  * Embedder that uses an external program (like sed or exiftool) to embed text
  * content and metadata into a given document.
@@ -105,7 +107,7 @@ public class ExternalEmbedder implements Embedder {
 
     /**
      * Gets the command to be run. This can include either of
-     * {@link #INPUT_FILE_TOKEN} or {@link #OUTPUT_FILE_TOKEN} if the command
+     * {@link ExternalParser#INPUT_FILE_TOKEN} or {@link ExternalParser#OUTPUT_FILE_TOKEN} if the command
      * needs filenames.
      *
      * @return
@@ -116,7 +118,7 @@ public class ExternalEmbedder implements Embedder {
 
     /**
      * Sets the command to be run. This can include either of
-     * {@link #INPUT_FILE_TOKEN} or {@link #OUTPUT_FILE_TOKEN} if the command
+     * {@link ExternalParser#INPUT_FILE_TOKEN} or {@link ExternalParser#OUTPUT_FILE_TOKEN} if the command
      * needs filenames.
      *
      * @see Runtime#exec(String[])
@@ -413,7 +415,7 @@ public class ExternalEmbedder implements Embedder {
             if (process.exitValue() != 0) {
                 throw new TikaException("There was an error executing the command line" +
                         "\nExecutable Command:\n\n" + cmd +
-                        "\nExecutable Error:\n\n" + stdErrOutputStream.toString("UTF-8"));
+                        "\nExecutable Error:\n\n" + stdErrOutputStream.toString(UTF_8.name()));
             }
         }
     }
