@@ -22,8 +22,10 @@ import java.io.IOException;
 import java.io.InputStream;
 
 import org.apache.tika.exception.EncryptedDocumentException;
+import org.apache.tika.exception.CorruptedFileException;
 import org.apache.tika.exception.TikaException;
 import org.apache.tika.io.CloseShieldInputStream;
+import org.apache.tika.io.IOExceptionWithCause;
 import org.apache.tika.io.TemporaryResources;
 import org.apache.tika.io.TikaInputStream;
 import org.apache.tika.metadata.Metadata;
@@ -91,8 +93,7 @@ public class ParsingEmbeddedDocumentExtractor implements EmbeddedDocumentExtract
         }
 
         // Use the delegate parser to parse this entry
-        TemporaryResources tmp = new TemporaryResources();
-        try {
+        try (TemporaryResources tmp = new TemporaryResources()) {
             final TikaInputStream newStream = TikaInputStream.get(new CloseShieldInputStream(stream), tmp);
             if (stream instanceof TikaInputStream) {
                 final Object container = ((TikaInputStream) stream).getOpenContainer();
@@ -107,11 +108,11 @@ public class ParsingEmbeddedDocumentExtractor implements EmbeddedDocumentExtract
         } catch (EncryptedDocumentException ede) {
             // TODO: can we log a warning that we lack the password?
             // For now, just skip the content
+        } catch (CorruptedFileException e) {
+            throw new IOExceptionWithCause(e);
         } catch (TikaException e) {
             // TODO: can we log a warning somehow?
             // Could not parse the entry, just skip the content
-        } finally {
-            tmp.close();
         }
 
         if(outputHtml) {
