@@ -26,4 +26,9 @@ public interface TikaMetadataKeys {
     String PROTECTED = "protected";
 
     String EMBEDDED_RELATIONSHIP_ID = "embeddedRelationshipId";
+
+    String EMBEDDED_STORAGE_CLASS_ID = "embeddedStorageClassId";
+
+    String EMBEDDED_RESOURCE_TYPE = "embeddedResourceType";
+
 }
