@@ -30,6 +30,7 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
+import java.util.concurrent.ConcurrentHashMap;
 
 import javax.xml.namespace.QName;
 
@@ -71,7 +72,7 @@ public final class MimeTypes implements Detector, Serializable {
     public static final String PLAIN_TEXT = "text/plain";
     
     /**
-     * Name of the {@link #xml xml} type, application/xml.
+     * Name of the {@link #xmlMimeType xml} type, application/xml.
      */
     public static final String XML = "application/xml";
 
@@ -79,13 +80,19 @@ public final class MimeTypes implements Detector, Serializable {
      * Root type, application/octet-stream.
      */
     private final MimeType rootMimeType;
+    private final List<MimeType> rootMimeTypeL;
 
     /**
      * Text type, text/plain.
      */
     private final MimeType textMimeType;
 
-    /*
+    /**
+     * html type, text/html
+     */
+    private final MimeType htmlMimeType;
+
+    /**
      * xml type, application/xml
      */
     private final MimeType xmlMimeType;
@@ -96,8 +103,7 @@ public final class MimeTypes implements Detector, Serializable {
     private final MediaTypeRegistry registry = new MediaTypeRegistry();
 
     /** All the registered MimeTypes indexed on their canonical names */
-    private final Map<MediaType, MimeType> types =
-        new HashMap<MediaType, MimeType>();
+    private final Map<MediaType, MimeType> types = new HashMap<>();
 
     /** The patterns matcher */
     private Patterns patterns = new Patterns(registry);
@@ -111,7 +117,10 @@ public final class MimeTypes implements Detector, Serializable {
     public MimeTypes() {
         rootMimeType = new MimeType(MediaType.OCTET_STREAM);
         textMimeType = new MimeType(MediaType.TEXT_PLAIN);
+        htmlMimeType = new MimeType(MediaType.TEXT_HTML);
         xmlMimeType = new MimeType(MediaType.APPLICATION_XML);
+        
+        rootMimeTypeL = Collections.singletonList(rootMimeType);
 
         add(rootMimeType);
         add(textMimeType);
@@ -157,7 +166,11 @@ public final class MimeTypes implements Detector, Serializable {
     /**
      * Returns the MIME type that best matches the given first few bytes
      * of a document stream. Returns application/octet-stream if no better
-     * match is found.
+     * match is found. 
+     * <p>
+     * If multiple matches are found, the best (highest priority) matching
+     * type is returned. If multiple matches are found with the same priority,
+     * then all of these are returned.
      * <p>
      * The given byte array is expected to be at least {@link #getMinLength()}
      * long, or shorter only if the document stream itself is shorter.
@@ -165,44 +178,68 @@ public final class MimeTypes implements Detector, Serializable {
      * @param data first few bytes of a document stream
      * @return matching MIME type
      */
-    private MimeType getMimeType(byte[] data) {
+    List<MimeType> getMimeType(byte[] data) {
         if (data == null) {
             throw new IllegalArgumentException("Data is missing");
         } else if (data.length == 0) {
             // See https://issues.apache.org/jira/browse/TIKA-483
-            return rootMimeType;
+            return rootMimeTypeL;
         }
 
         // Then, check for magic bytes
-        MimeType result = null;
+        List<MimeType> result = new ArrayList<MimeType>(1);
+        int currentPriority = -1;
         for (Magic magic : magics) {
-            if (magic.eval(data)) {
-                result = magic.getType();
+            if (currentPriority > 0 && currentPriority > magic.getPriority()) {
                 break;
             }
+            if (magic.eval(data)) {
+                result.add(magic.getType());
+                currentPriority = magic.getPriority();
+            }
         }
  
-        if (result != null) {
-            // When detecting generic XML (or possibly XHTML),
-            // extract the root element and match it against known types
-            if ("application/xml".equals(result.getName())
-                    || "text/html".equals(result.getName())) {
-                XmlRootExtractor extractor = new XmlRootExtractor();
+        if (!result.isEmpty()) {
+            for (int i=0; i<result.size(); i++) {
+                final MimeType matched = result.get(i);
+                
+                // When detecting generic XML (or possibly XHTML),
+                // extract the root element and match it against known types
+                if ("application/xml".equals(matched.getName())
+                        || "text/html".equals(matched.getName())) {
+                    XmlRootExtractor extractor = new XmlRootExtractor();
 
-                QName rootElement = extractor.extractRootElement(data);
-                if (rootElement != null) {
-                    for (MimeType type : xmls) {
-                        if (type.matchesXML(
-                                rootElement.getNamespaceURI(),
-                                rootElement.getLocalPart())) {
-                            result = type;
-                            break;
+                    QName rootElement = extractor.extractRootElement(data);
+                    if (rootElement != null) {
+                        for (MimeType type : xmls) {
+                            if (type.matchesXML(
+                                    rootElement.getNamespaceURI(),
+                                    rootElement.getLocalPart())) {
+                                result.set(i, type);
+                                break;
+                            }
+                        }
+                    } else if ("application/xml".equals(matched.getName())) {
+                        // Our XML magic is higher than our HTML magic
+                        // So, if we got here, we might have a HTML file that's
+                        //  invalid XML. So, try our HTML magics explicitly (TIKA-2419)
+                        boolean isHTML = false;
+                        for (Magic magic : magics) {
+                            if (! magic.getType().equals(htmlMimeType)) continue;
+                            if (magic.eval(data)) {
+                                isHTML = true;
+                                break;
+                            }
+                        }
+                        
+                        // Otherwise, downgrade from application/xml to text/plain
+                        //  since the document seems not to be well-formed.
+                        if (isHTML) {
+                            result.set(i, htmlMimeType);
+                        } else {
+                            result.set(i, textMimeType);
                         }
                     }
-                } else if ("application/xml".equals(result.getName())) {
-                    // Downgrade from application/xml to text/plain since
-                    // the document seems not to be well-formed.
-                    result = textMimeType;
                 }
             }
             return result;
@@ -212,9 +249,10 @@ public final class MimeTypes implements Detector, Serializable {
         try {
             TextDetector detector = new TextDetector(getMinLength());
             ByteArrayInputStream stream = new ByteArrayInputStream(data);
-            return forName(detector.detect(stream, new Metadata()).toString());
+            MimeType type = forName(detector.detect(stream, new Metadata()).toString());
+            return Collections.singletonList(type);
         } catch (Exception e) {
-            return rootMimeType;
+            return rootMimeTypeL;
         }
     }
 
@@ -231,7 +269,7 @@ public final class MimeTypes implements Detector, Serializable {
      * @return first {@link #getMinLength()} (or fewer) bytes of the stream
      * @throws IOException if the stream can not be read
      */
-    private byte[] readMagicHeader(InputStream stream) throws IOException {
+    byte[] readMagicHeader(InputStream stream) throws IOException {
         if (stream == null) {
             throw new IllegalArgumentException("InputStream is missing");
         }
@@ -287,20 +325,37 @@ public final class MimeTypes implements Detector, Serializable {
     }
 
     /**
-     * Returns the registered media type with the given name (or alias).
+     * Returns the registered, normalised media type with the given name (or alias).
      * 
-     * Unlike {@link #forName(String)}, this function will *not* create a new
-     * MimeType and register it
+     * <p>Unlike {@link #forName(String)}, this function will <em>not<em> create a 
+     * new MimeType and register it. Instead, <code>null</code> will be returned if 
+     * there is no definition available for the given name.
+     *
+     * <p>Also, unlike {@link #forName(String)}, this function may return a
+     * mime type that has fewer parameters than were included in the supplied name.
+     * If the registered mime type has parameters (e.g. 
+     * <code>application/dita+xml;format=map</code>), then those will be maintained.  
+     * However, if the supplied name has paramenters that the <em>registered</em> mime
+     * type does not (e.g. <code>application/xml; charset=UTF-8</code> as a name, 
+     * compared to just <code>application/xml</code> for the type in the registry), 
+     * then those parameters will not be included in the returned type.
      *
      * @param name media type name (case-insensitive)
-     * @return the registered media type with the given name or alias
+     * @return the registered media type with the given name or alias, or null if not found
      * @throws MimeTypeException if the given media type name is invalid
      */
     public MimeType getRegisteredMimeType(String name) throws MimeTypeException {
         MediaType type = MediaType.parse(name);
         if (type != null) {
             MediaType normalisedType = registry.normalize(type);
-            return types.get(normalisedType);
+            MimeType candidate = types.get(normalisedType);
+            if (candidate != null) {
+                return candidate;
+            }
+            if (normalisedType.hasParameters()) {
+                return types.get(normalisedType.getBaseType());
+            }
+            return null;
         } else {
             throw new MimeTypeException("Invalid media type name: " + name);
         }
@@ -370,7 +425,6 @@ public final class MimeTypes implements Detector, Serializable {
      *
      * @return the minimum length of data to provide.
      * @see #getMimeType(byte[])
-     * @see #getMimeType(String, byte[])
      */
     public int getMinLength() {
         // This needs to be reasonably large to be able to correctly detect
@@ -429,14 +483,14 @@ public final class MimeTypes implements Detector, Serializable {
      */
     public MediaType detect(InputStream input, Metadata metadata)
             throws IOException {
-        MediaType type = MediaType.OCTET_STREAM;
+        List<MimeType> possibleTypes = null;
 
         // Get type based on magic prefix
         if (input != null) {
             input.mark(getMinLength());
             try {
                 byte[] prefix = readMagicHeader(input);
-                type = getMimeType(prefix).getType();
+                possibleTypes = getMimeType(prefix);
             } finally {
                 input.reset();
             }
@@ -446,10 +500,13 @@ public final class MimeTypes implements Detector, Serializable {
         String resourceName = metadata.get(Metadata.RESOURCE_NAME_KEY);
         if (resourceName != null) {
             String name = null;
+            boolean isHttp = false;
 
             // Deal with a URI or a path name in as the resource  name
             try {
                 URI uri = new URI(resourceName);
+                String scheme = uri.getScheme();
+                isHttp = scheme != null && scheme.startsWith("http"); // http or https
                 String path = uri.getPath();
                 if (path != null) {
                     int slash = path.lastIndexOf('/');
@@ -462,9 +519,14 @@ public final class MimeTypes implements Detector, Serializable {
             }
 
             if (name != null) {
-                MediaType hint = getMimeType(name).getType();
-                if (registry.isSpecializationOf(hint, type)) {
-                    type = hint;
+                MimeType hint = getMimeType(name);
+
+                // For server-side scripting languages, we cannot rely on the filename to detect the mime type
+                if (!(isHttp && hint.isInterpreted())) {
+                    // If we have some types based on mime magic, try to specialise
+                    //  and/or select the type based on that
+                    // Otherwise, use the type identified from the name
+                    possibleTypes = applyHint(possibleTypes, hint);
                 }
             }
         }
@@ -473,16 +535,42 @@ public final class MimeTypes implements Detector, Serializable {
         String typeName = metadata.get(Metadata.CONTENT_TYPE);
         if (typeName != null) {
             try {
-                MediaType hint = forName(typeName).getType();
-                if (registry.isSpecializationOf(hint, type)) {
-                    type = hint;
-                }
+                MimeType hint = forName(typeName);
+                possibleTypes = applyHint(possibleTypes, hint);
             } catch (MimeTypeException e) {
                 // Malformed type name, ignore
             }
         }
 
-        return type;
+        if (possibleTypes == null || possibleTypes.isEmpty()) {
+            // Report that we don't know what it is
+            return MediaType.OCTET_STREAM;
+        } else {
+            return possibleTypes.get(0).getType();
+        }
+    }
+    /**
+     * Use the MimeType hint to try to clarify or specialise the current
+     *  possible types list.
+     * If the hint is a specialised form, use that instead
+     * If there are multiple possible types, use the hint to select one
+     */
+    private List<MimeType> applyHint(List<MimeType> possibleTypes, MimeType hint) {
+        if (possibleTypes == null || possibleTypes.isEmpty()) {
+            return Collections.singletonList(hint);
+        } else {
+            for (int i=0; i<possibleTypes.size(); i++) {
+                final MimeType type = possibleTypes.get(i);
+                if (hint.equals(type) ||
+                    registry.isSpecializationOf(hint.getType(), type.getType())) {
+                    // Use just this type
+                    return Collections.singletonList(hint);
+                }
+            }
+        }
+        
+        // Hint didn't help, sorry
+        return possibleTypes;
     }
 
     private static MimeTypes DEFAULT_TYPES = null;
@@ -502,7 +590,7 @@ public final class MimeTypes implements Detector, Serializable {
      * Get the default MimeTypes. This includes all the built-in
      * media types, and any custom override ones present.
      * 
-     * @param ClassLoader to use, if not the default
+     * @param classLoader to use, if not the default
      * @return MimeTypes default type registry
      */
     public static synchronized MimeTypes getDefaultMimeTypes(ClassLoader classLoader) {
