@@ -24,11 +24,14 @@ import javax.xml.parsers.DocumentBuilder;
 import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.PrintStream;
 import java.lang.reflect.Constructor;
+import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.Date;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Random;
 import java.util.Set;
 
 import org.apache.tika.exception.TikaException;
@@ -64,11 +67,18 @@ import org.xml.sax.SAXException;
 
 public class MockParser extends AbstractParser {
 
-    private static final long serialVersionUID = 1L;
 
+    private static final long serialVersionUID = 1L;
+    private static PrintStream ORIG_STDERR;
+    private static PrintStream ORIG_STDOUT;
+    static {
+        ORIG_STDERR = System.err;
+        ORIG_STDOUT = System.out;
+    }
+    private final Random random = new Random();
     @Override
     public Set<MediaType> getSupportedTypes(ParseContext context) {
-        Set<MediaType> types = new HashSet<MediaType>();
+        Set<MediaType> types = new HashSet<>();
         MediaType type = MediaType.application("mock+xml");
         types.add(type);
         return types;
@@ -78,6 +88,9 @@ public class MockParser extends AbstractParser {
     public void parse(InputStream stream, ContentHandler handler,
                       Metadata metadata, ParseContext context) throws IOException,
             SAXException, TikaException {
+        if (Thread.currentThread().isInterrupted()) {
+            throw new TikaException("interrupted", new InterruptedException());
+        }
         Document doc = null;
         try {
             DocumentBuilder docBuilder = context.getDocumentBuilder();
@@ -121,6 +134,10 @@ public class MockParser extends AbstractParser {
             handleEmbedded(action, xhtml, context);
         } else if ("throwIllegalChars".equals(name)) {
             throwIllegalChars();
+        } else if ("system_exit".equals(name)) {
+            System.exit(1);
+        } else if ("thread_interrupt".equals(name)) {
+            Thread.currentThread().interrupt();
         } else {
             throw new IllegalArgumentException("Didn't recognize mock action: "+name);
         }
@@ -176,16 +193,43 @@ public class MockParser extends AbstractParser {
         return extractor;
     }
 
-    private void print(Node action, String name) {
+    private void print(Node action, String name) throws IOException {
         String content = action.getTextContent();
-        if ("print_out".equals(name)) {
-            System.out.println(content);
-        } else if ("print_err".equals(name)) {
-            System.err.println(content);
+        boolean isStatic = (action.getAttributes().getNamedItem("static") == null) ? false : true;
+        Node rand = action.getAttributes().getNamedItem("random");
+        int randLength = -1;
+        if (rand != null) {
+            randLength = Integer.parseInt(rand.getNodeValue());
+        }
+        byte[] contentBytes = getBytes(content, randLength);
+        if (isStatic) {
+            if ("print_out".equals(name)) {
+                ORIG_STDOUT.write(contentBytes);
+            } else if ("print_err".equals(name)) {
+                ORIG_STDERR.write(contentBytes);
+            } else {
+                throw new IllegalArgumentException("must be print_out or print_err");
+            }
         } else {
-            throw new IllegalArgumentException("must be print_out or print_err");
+            if ("print_out".equals(name)) {
+                System.out.write(contentBytes);
+            } else if ("print_err".equals(name)) {
+                System.err.write(contentBytes);
+            } else {
+                throw new IllegalArgumentException("must be print_out or print_err");
+            }
         }
     }
+
+    private byte[] getBytes(String content, int randLength) {
+        if (randLength < 0) {
+            return content.getBytes(StandardCharsets.UTF_8);
+        }
+        byte[] bytes = new byte[randLength];
+        random.nextBytes(bytes);
+        return bytes;
+    }
+
     private void hang(Node action) {
         boolean interruptible = true;
         boolean heavy = false;
@@ -256,7 +300,7 @@ public class MockParser extends AbstractParser {
         }
     }
 
-    private void write(Node action, XHTMLContentHandler xhtml) throws SAXException {
+    protected void write(Node action, XHTMLContentHandler xhtml) throws SAXException {
         NamedNodeMap attrs = action.getAttributes();
         Node eNode = attrs.getNamedItem("element");
         String elementType = "p";
@@ -305,7 +349,7 @@ public class MockParser extends AbstractParser {
     }
 
     private void kabOOM() {
-        List<int[]> ints = new ArrayList<int[]>();
+        List<int[]> ints = new ArrayList<>();
 
         while (true) {
             int[] intArr = new int[32000];
@@ -316,16 +360,19 @@ public class MockParser extends AbstractParser {
     private void hangHeavy(long maxMillis, long pulseCheckMillis, boolean interruptible) {
         //do some heavy computation and occasionally check for
         //whether time has exceeded maxMillis (see TIKA-1132 for inspiration)
-        //or whether the thread was interrupted
+        //or whether the thread was interrupted.
+        //By creating a new Date in the inner loop, we're also intentionally
+        //triggering the gc most likely.
         long start = new Date().getTime();
-        int lastChecked = 0;
+        long lastChecked = start;
         while (true) {
             for (int i = 1; i < Integer.MAX_VALUE; i++) {
                 for (int j = 1; j < Integer.MAX_VALUE; j++) {
                     double div = (double) i / (double) j;
-                    lastChecked++;
-                    if (lastChecked > pulseCheckMillis) {
-                        lastChecked = 0;
+
+                    long elapsedSinceLastCheck = new Date().getTime()-lastChecked;
+                    if (elapsedSinceLastCheck > pulseCheckMillis) {
+                        lastChecked = new Date().getTime();
                         if (interruptible && Thread.currentThread().isInterrupted()) {
                             return;
                         }
@@ -340,7 +387,7 @@ public class MockParser extends AbstractParser {
     }
 
     private void sleep(long maxMillis, boolean isInterruptible) {
-        long start = new Date().getTime();
+        long start = System.currentTimeMillis();
         long millisRemaining = maxMillis;
         while (true) {
             try {
@@ -350,7 +397,7 @@ public class MockParser extends AbstractParser {
                     return;
                 }
             }
-            long elapsed = new Date().getTime()-start;
+            long elapsed = System.currentTimeMillis()-start;
             millisRemaining = maxMillis - elapsed;
             if (millisRemaining <= 0) {
                 break;
