@@ -16,28 +16,39 @@
  */
 package org.apache.tika.parser;
 
+import org.apache.tika.TikaTest;
+import org.apache.tika.config.TikaConfig;
+import org.apache.tika.detect.Detector;
+import org.apache.tika.exception.TikaException;
+import org.apache.tika.exception.ZeroByteFileException;
+import org.apache.tika.io.TikaInputStream;
+import org.apache.tika.metadata.Metadata;
+import org.apache.tika.metadata.TikaCoreProperties;
+import org.apache.tika.metadata.XMPDM;
+import org.apache.tika.mime.MediaType;
+import org.apache.tika.sax.BodyContentHandler;
+import org.gagravarr.tika.FlacParser;
+import org.gagravarr.tika.OpusParser;
+import org.gagravarr.tika.VorbisParser;
+import org.junit.Test;
+import org.xml.sax.ContentHandler;
+
+import java.io.ByteArrayInputStream;
+import java.io.ByteArrayOutputStream;
+import java.io.IOException;
+import java.io.InputStream;
+import java.util.HashSet;
+import java.util.Set;
+import java.util.zip.ZipEntry;
+import java.util.zip.ZipOutputStream;
+
+import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
-import java.io.ByteArrayInputStream;
-import java.io.IOException;
-import java.io.InputStream;
-import java.util.HashSet;
-import java.util.Set;
-
-import org.apache.tika.config.TikaConfig;
-import org.apache.tika.detect.Detector;
-import org.apache.tika.exception.TikaException;
-import org.apache.tika.metadata.Metadata;
-import org.apache.tika.metadata.XMPDM;
-import org.apache.tika.mime.MediaType;
-import org.apache.tika.sax.BodyContentHandler;
-import org.junit.Test;
-import org.xml.sax.ContentHandler;
-
-public class AutoDetectParserTest {
+public class AutoDetectParserTest extends TikaTest {
     private TikaConfig tika = TikaConfig.getDefaultConfig();
 
     // Easy to read constants for the MIME types:
@@ -56,12 +67,13 @@ public class AutoDetectParserTest {
     private static final String WORD       = "application/msword";
     private static final String XML        = "application/xml";
     private static final String RSS        = "application/rss+xml";
-    private static final String BMP        = "image/x-ms-bmp";
+    private static final String BMP        = "image/bmp";
     private static final String GIF        = "image/gif";
     private static final String JPEG       = "image/jpeg";
     private static final String PNG        = "image/png";
     private static final String OGG_VORBIS = "audio/vorbis";
-    private static final String OGG_FLAC   = "audio/x-flac";
+    private static final String OGG_OPUS   = "audio/opus";
+    private static final String OGG_FLAC   = "audio/x-oggflac"; 
     private static final String FLAC_NATIVE= "audio/x-flac";
     private static final String OPENOFFICE
             = "application/vnd.oasis.opendocument.text";
@@ -73,16 +85,11 @@ public class AutoDetectParserTest {
      * @throws IOException
      */
     private void assertAutoDetect(TestParams tp) throws Exception {
-
-        InputStream input =
-            AutoDetectParserTest.class.getResourceAsStream(tp.resourceRealName);
-
-        if (input == null) {
-            fail("Could not open stream from specified resource: "
-                    + tp.resourceRealName);
-        }
-
-        try {
+        try (InputStream input = AutoDetectParserTest.class.getResourceAsStream(tp.resourceRealName)) {
+            if (input == null) {
+                fail("Could not open stream from specified resource: "
+                        + tp.resourceRealName);
+            }
             Metadata metadata = new Metadata();
             metadata.set(Metadata.RESOURCE_NAME_KEY, tp.resourceStatedName);
             metadata.set(Metadata.CONTENT_TYPE, tp.statedType);
@@ -93,11 +100,9 @@ public class AutoDetectParserTest {
                     tp.realType, metadata.get(Metadata.CONTENT_TYPE));
 
             if (tp.expectedContentFragment != null) {
-               assertTrue("Expected content not found: " + tp,
-                       handler.toString().contains(tp.expectedContentFragment));
+                assertTrue("Expected content not found: " + tp,
+                        handler.toString().contains(tp.expectedContentFragment));
             }
-        } finally {
-            input.close();
         }
     }
 
@@ -236,7 +241,7 @@ public class AutoDetectParserTest {
 
     @Test
     public void testRss() throws Exception {
-        assertAutoDetect("/test-documents/rsstest.rss", "feed", RSS, "application/rss+xml", "Sample RSS File for Junit test");
+        assertAutoDetect("/test-documents/rsstest_091.rss", "feed", RSS, "application/rss+xml", "Sample RSS File for Junit test");
     }
     
     @Test
@@ -254,80 +259,111 @@ public class AutoDetectParserTest {
      */
     @Test
     public void testZipBombPrevention() throws Exception {
-        InputStream tgz = AutoDetectParserTest.class.getResourceAsStream(
-                "/test-documents/TIKA-216.tgz");
-        try {
+        try (InputStream tgz = AutoDetectParserTest.class.getResourceAsStream(
+                "/test-documents/TIKA-216.tgz")) {
             Metadata metadata = new Metadata();
             ContentHandler handler = new BodyContentHandler(-1);
             new AutoDetectParser(tika).parse(tgz, handler, metadata);
             fail("Zip bomb was not detected");
         } catch (TikaException e) {
             // expected
-        } finally {
-            tgz.close();
         }
-    
     }
 
     /**
-     * Test to ensure that the Vorbis and FLAC parsers have been correctly
-     *  included, and are available
+     * Make sure XML parse errors don't trigger ZIP bomb detection.
+     *
+     * @see <a href="https://issues.apache.org/jira/browse/TIKA-1322">TIKA-1322</a>
+     */
+    @Test
+    public void testNoBombDetectedForInvalidXml() throws Exception {
+        // create zip with ten empty / invalid XML files, 1.xml .. 10.xml
+        ByteArrayOutputStream baos = new ByteArrayOutputStream();
+        ZipOutputStream zos = new ZipOutputStream(baos);
+        for (int i = 1; i <= 10; i++) {
+            zos.putNextEntry(new ZipEntry(i + ".xml"));
+            zos.closeEntry();
+        }
+        zos.finish();
+        zos.close();
+        new AutoDetectParser(tika).parse(new ByteArrayInputStream(baos.toByteArray()), new BodyContentHandler(-1),
+                new Metadata());
+    }
+
+    /**
+     * Test to ensure that the Ogg Audio parsers (Vorbis, Opus, Flac etc)
+     *  have been correctly included, and are available
      */
     @SuppressWarnings("deprecation")
     @Test
-    public void testVorbisFlac() throws Exception {
+    public void testOggFlacAudio() throws Exception {
        // The three test files should all have similar test data
        String[] testFiles = new String[] {
-             "testVORBIS.ogg", "testFLAC.oga", "testFLAC.flac"
+             "testVORBIS.ogg", "testFLAC.flac", "testFLAC.oga",
+             "testOPUS.opus"
        };
-       String[] mimetypes = new String[] {
-             OGG_VORBIS, OGG_FLAC, FLAC_NATIVE
+       MediaType[] mediaTypes = new MediaType[] {
+               MediaType.parse(OGG_VORBIS), MediaType.parse(FLAC_NATIVE),
+               MediaType.parse(OGG_FLAC), MediaType.parse(OGG_OPUS)
        };
        
+       // Check we can load the parsers, and they claim to do the right things
+       VorbisParser vParser = new VorbisParser();
+       assertNotNull("Parser not found for " + mediaTypes[0], 
+                     vParser.getSupportedTypes(new ParseContext()));
+       
+       FlacParser fParser = new FlacParser();
+       assertNotNull("Parser not found for " + mediaTypes[1], 
+                     fParser.getSupportedTypes(new ParseContext()));
+       assertNotNull("Parser not found for " + mediaTypes[2], 
+                     fParser.getSupportedTypes(new ParseContext()));
+       
+       OpusParser oParser = new OpusParser();
+       assertNotNull("Parser not found for " + mediaTypes[3], 
+                     oParser.getSupportedTypes(new ParseContext()));
+       
        // Check we found the parser
        CompositeParser parser = (CompositeParser)tika.getParser();
-       for (String type : mimetypes) {
-          MediaType mt = MediaType.parse(type);
-          assertNotNull("Parser not found for " + type, parser.getParsers().get(mt) );
+       for (MediaType mt : mediaTypes) {
+          assertNotNull("Parser not found for " + mt, parser.getParsers().get(mt) );
        }
        
        // Have each file parsed, and check
        for (int i=0; i<testFiles.length; i++) {
-          String file = testFiles[i];
-          InputStream input = AutoDetectParserTest.class.getResourceAsStream(
-                "/test-documents/"+file);
+           String file = testFiles[i];
+           try (InputStream input = AutoDetectParserTest.class.getResourceAsStream(
+                   "/test-documents/" + file)) {
+               if (input == null) {
+                   fail("Could not find test file " + file);
+               }
+               Metadata metadata = new Metadata();
+               ContentHandler handler = new BodyContentHandler();
+               new AutoDetectParser(tika).parse(input, handler, metadata);
 
-          if (input == null) {
-             fail("Could not find test file " + file);
-          }
-          
-          try {
-             Metadata metadata = new Metadata();
-             ContentHandler handler = new BodyContentHandler();
-             new AutoDetectParser(tika).parse(input, handler, metadata);
+               assertEquals("Incorrect content type for " + file,
+                       mediaTypes[i].toString(), metadata.get(Metadata.CONTENT_TYPE));
 
-             assertEquals("Incorrect content type for " + file,
-                   mimetypes[i], metadata.get(Metadata.CONTENT_TYPE));
+               // Check some of the common metadata
+               // Old style metadata
+               assertEquals("Test Artist", metadata.get(Metadata.AUTHOR));
+               assertEquals("Test Title", metadata.get(Metadata.TITLE));
+               // New style metadata
+               assertEquals("Test Artist", metadata.get(TikaCoreProperties.CREATOR));
+               assertEquals("Test Title", metadata.get(TikaCoreProperties.TITLE));
 
-             // Check some of the common metadata
-             assertEquals("Test Artist", metadata.get(Metadata.AUTHOR));
-             assertEquals("Test Title", metadata.get(Metadata.TITLE));
-//             assertEquals("Test Artist", metadata.get(TikaCoreProperties.AUTHOR));
-//             assertEquals("Test Title", metadata.get(TikaCoreProperties.TITLE));
-             
-             // Check some of the XMPDM metadata
-             assertEquals("Test Album", metadata.get(XMPDM.ALBUM));
-             assertEquals("Test Artist", metadata.get(XMPDM.ARTIST));
-             assertEquals("Stereo", metadata.get(XMPDM.AUDIO_CHANNEL_TYPE));
-             assertEquals("44100", metadata.get(XMPDM.AUDIO_SAMPLE_RATE));
-             
-             // Check some of the text
-             String content = handler.toString();
-             assertTrue(content.contains("Test Title"));
-             assertTrue(content.contains("Test Artist"));
-          } finally {
-             input.close();
-          }
+               // Check some of the XMPDM metadata
+               if (!file.endsWith(".opus")) {
+                   assertEquals("Test Album", metadata.get(XMPDM.ALBUM));
+               }
+               assertEquals("Test Artist", metadata.get(XMPDM.ARTIST));
+               assertEquals("Stereo", metadata.get(XMPDM.AUDIO_CHANNEL_TYPE));
+               assertEquals("44100", metadata.get(XMPDM.AUDIO_SAMPLE_RATE));
+
+               // Check some of the text
+               String content = handler.toString();
+               assertTrue(content.contains("Test Title"));
+               assertTrue(content.contains("Test Artist"));
+           }
        }
     }
     
@@ -340,13 +376,42 @@ public class AutoDetectParserTest {
     public void testSpecificParserList() throws Exception {
         AutoDetectParser parser = new AutoDetectParser(new MyDetector(), new MyParser());
         
-        InputStream is = new ByteArrayInputStream("test".getBytes());
+        InputStream is = new ByteArrayInputStream("test".getBytes(UTF_8));
         Metadata metadata = new Metadata();
         parser.parse(is, new BodyContentHandler(), metadata, new ParseContext());
         
         assertEquals("value", metadata.get("MyParser"));
     }
 
+    @Test
+    public void testZeroByteFileException() throws Exception {
+        String[] exts = new String[]{
+                "xls",
+                "doc",
+                "pdf",
+                "rtf"
+        };
+
+        String[] mimes = new String[]{
+                EXCEL,
+                WORD,
+                PDF,
+                RTF
+        };
+
+        for (int i = 0; i < exts.length; i++) {
+            Metadata m = new Metadata();
+            m.set(Metadata.RESOURCE_NAME_KEY, "file." + exts[i]);
+            try {
+                getXML(TikaInputStream.get(new byte[0]), new AutoDetectParser(), m);
+                fail("should have thrown zero byte exception");
+            } catch (ZeroByteFileException e) {
+
+            }
+            assertEquals(mimes[i], m.get(Metadata.CONTENT_TYPE));
+        }
+    }
+
     private static final MediaType MY_MEDIA_TYPE = new MediaType("application", "x-myparser");
     
     /**
