@@ -31,35 +31,36 @@ import org.xml.sax.ContentHandler;
 public class FeedParserTest {
     @Test
     public void testRSSParser() throws Exception {
-        InputStream input = FeedParserTest.class
-                .getResourceAsStream("/test-documents/rsstest.rss");
-        try {
-            Metadata metadata = new Metadata();
-            ContentHandler handler = new BodyContentHandler();
-            ParseContext context = new ParseContext();
-
-            new FeedParser().parse(input, handler, metadata, context);
-
-            String content = handler.toString();
-            assertFalse(content == null);
-
-            assertEquals("Sample RSS File for Junit test",
-                    metadata.get(TikaCoreProperties.DESCRIPTION));
-            assertEquals("TestChannel", metadata.get(TikaCoreProperties.TITLE));
-
-            // TODO find a way of testing the paragraphs and anchors
-
-        } finally {
-            input.close();
+        // These RSS files should have basically the same contents,
+        //  represented in the various RSS format versions
+        for (String rssFile : new String[] {
+                "/test-documents/rsstest_091.rss",
+                "/test-documents/rsstest_20.rss"
+        }) {
+            try (InputStream input = FeedParserTest.class.getResourceAsStream(rssFile)) {
+                Metadata metadata = new Metadata();
+                ContentHandler handler = new BodyContentHandler();
+                ParseContext context = new ParseContext();
+    
+                new FeedParser().parse(input, handler, metadata, context);
+    
+                String content = handler.toString();
+                assertFalse(content == null);
+    
+                assertEquals("Sample RSS File for Junit test",
+                        metadata.get(TikaCoreProperties.DESCRIPTION));
+                assertEquals("TestChannel", metadata.get(TikaCoreProperties.TITLE));
+    
+                // TODO find a way of testing the paragraphs and anchors
+            }
         }
     }
 
 
     @Test
     public void testAtomParser() throws Exception {
-        InputStream input = FeedParserTest.class
-                .getResourceAsStream("/test-documents/testATOM.atom");
-        try {
+        try (InputStream input = FeedParserTest.class.getResourceAsStream(
+                "/test-documents/testATOM.atom")) {
             Metadata metadata = new Metadata();
             ContentHandler handler = new BodyContentHandler();
             ParseContext context = new ParseContext();
@@ -74,8 +75,6 @@ public class FeedParserTest {
             assertEquals("Test Atom Feed", metadata.get(TikaCoreProperties.TITLE));
 
             // TODO Check some more
-        } finally {
-            input.close();
         }
     }
 
