@@ -71,8 +71,11 @@ import org.apache.tika.parser.AutoDetectParser;
 import org.apache.tika.parser.ParseContext;
 import org.apache.tika.parser.Parser;
 import org.apache.tika.parser.RecursiveParserWrapper;
+import org.apache.tika.sax.AbstractRecursiveParserWrapperHandler;
+import org.apache.tika.sax.BasicContentHandlerFactory;
 import org.apache.tika.sax.BodyContentHandler;
 import org.apache.tika.sax.LinkContentHandler;
+import org.apache.tika.sax.RecursiveParserWrapperHandler;
 import org.apache.tika.sax.TeeContentHandler;
 import org.ccil.cowan.tagsoup.HTMLSchema;
 import org.ccil.cowan.tagsoup.Schema;
@@ -739,7 +742,7 @@ public class HtmlParserTest extends TikaTest {
         String content = sw.toString();
 
         // Should have <html>, <head>, <title>, <body> elements
-        assertTrue(Pattern.matches("(?s).*<html xmlns=\"http://www.w3.org/1999/xhtml\">.*</html>.*$", content));
+        assertTrue(Pattern.matches("(?s).*<html xmlns=\"http://www.w3.org/1999/xhtml\" lang=\"en\">.*</html>.*$", content));
         assertTrue(Pattern.matches("(?s).*<head>.*</head>.*$", content));
         assertTrue(Pattern.matches("(?s).*<title>Title</title>.*$", content));
         assertTrue(Pattern.matches("(?s).*<body>.*</body>.*$", content));
@@ -872,6 +875,25 @@ public class HtmlParserTest extends TikaTest {
         assertTrue(Pattern.matches("\tone\n\n", result));
     }
 
+    /**
+     * Test case for Tika-2100
+     * @see <a href="https://issues.apache.org/jira/browse/TIKA-2100">TIKA-2100</a>
+     */
+    @Test
+    public void testHtmlLanguage() throws Exception {
+        final String html = "<html lang=\"fr\"></html>";
+
+        StringWriter sw = new StringWriter();
+        Metadata metadata = new Metadata();
+        new HtmlParser().parse(
+                new ByteArrayInputStream(html.getBytes(UTF_8)),
+                makeHtmlTransformer(sw), metadata, new ParseContext());
+
+        assertEquals("fr", metadata.get(Metadata.CONTENT_LANGUAGE));
+        assertTrue("Missing HTML lang attribute",
+                Pattern.matches("(?s)<html[^>]* lang=\"fr\".*", sw.toString()));
+    }
+
     /**
      * Test case for TIKA-961
      *
@@ -1244,13 +1266,15 @@ public class HtmlParserTest extends TikaTest {
     public void testExtractScript() throws Exception {
         HtmlParser p = new HtmlParser();
         p.setExtractScripts(true);
+        //TIKA-2550 -- make absolutely sure that macros are still extracted
+        //with the ToTextHandler
         List<Metadata> metadataList = getRecursiveMetadata("testHTMLGoodScript.html",
-                p);
+                p, BasicContentHandlerFactory.HANDLER_TYPE.TEXT);
         assertEquals(2, metadataList.size());
         assertEquals("MACRO", metadataList.get(1).get(TikaCoreProperties.EMBEDDED_RESOURCE_TYPE));
         assertContains("cool",
-                metadataList.get(1).get(RecursiveParserWrapper.TIKA_CONTENT));
-        assertNotContained("cool", metadataList.get(0).get(RecursiveParserWrapper.TIKA_CONTENT));
+                metadataList.get(1).get(AbstractRecursiveParserWrapperHandler.TIKA_CONTENT));
+        assertNotContained("cool", metadataList.get(0).get(AbstractRecursiveParserWrapperHandler.TIKA_CONTENT));
     }
 
     @Test
@@ -1263,8 +1287,8 @@ public class HtmlParserTest extends TikaTest {
         assertEquals(2, metadataList.size());
         assertEquals("MACRO", metadataList.get(1).get(TikaCoreProperties.EMBEDDED_RESOURCE_TYPE));
         assertContains("cool",
-                metadataList.get(1).get(RecursiveParserWrapper.TIKA_CONTENT));
-        assertNotContained("cool", metadataList.get(0).get(RecursiveParserWrapper.TIKA_CONTENT));
+                metadataList.get(1).get(AbstractRecursiveParserWrapperHandler.TIKA_CONTENT));
+        assertNotContained("cool", metadataList.get(0).get(AbstractRecursiveParserWrapperHandler.TIKA_CONTENT));
 
     }
 
@@ -1272,7 +1296,7 @@ public class HtmlParserTest extends TikaTest {
     public void testDataURI() throws Exception {
         List<Metadata> metadataList = getRecursiveMetadata("testHTML_embedded_img.html");
         assertEquals(2, metadataList.size());
-        String content = metadataList.get(0).get(RecursiveParserWrapper.TIKA_CONTENT);
+        String content = metadataList.get(0).get(AbstractRecursiveParserWrapperHandler.TIKA_CONTENT);
         assertContains("some content", content);
         //make sure that you've truncated the data: value
         assertContains("src=\"data:\"", content);
@@ -1290,7 +1314,7 @@ public class HtmlParserTest extends TikaTest {
         Parser p = new AutoDetectParser(tikaConfig);
         List<Metadata> metadataList = getRecursiveMetadata("testHTML_embedded_img_in_js.html", p);
         assertEquals(3, metadataList.size());
-        String content = metadataList.get(0).get(RecursiveParserWrapper.TIKA_CONTENT);
+        String content = metadataList.get(0).get(AbstractRecursiveParserWrapperHandler.TIKA_CONTENT);
         assertContains("some content", content);
         Metadata imgMetadata = metadataList.get(1);
         assertEquals("image/jpeg", imgMetadata.get(Metadata.CONTENT_TYPE));
@@ -1395,4 +1419,26 @@ public class HtmlParserTest extends TikaTest {
                 getXML("testHTML_charset_utf16le.html").xml);
 
     }
+
+    @Test
+    public void testSkippingDataURIInScriptNode() throws Exception {
+        //TIKA-2759 skip data: uri element if inside a script
+        //default behavior
+        List<Metadata> metadataList = getRecursiveMetadata("testHTML_embedded_data_uri_js.html");
+        assertEquals(1, metadataList.size());
+        assertNotContained("alert( 'Hello, world!' );",
+                metadataList.get(0).get(RecursiveParserWrapperHandler.TIKA_CONTENT));
+
+        //make sure to include it if a user wants scripts to be extracted
+        InputStream is = getClass().getResourceAsStream("/org/apache/tika/parser/html/tika-config.xml");
+        assertNotNull(is);
+        TikaConfig tikaConfig = new TikaConfig(is);
+        Parser p = new AutoDetectParser(tikaConfig);
+        metadataList = getRecursiveMetadata("testHTML_embedded_data_uri_js.html", p);
+        assertEquals(2, metadataList.size());
+        assertContains("alert( 'Hello, world!' );",
+                metadataList.get(1).get(RecursiveParserWrapperHandler.TIKA_CONTENT));
+
+
+    }
 }
