@@ -30,6 +30,7 @@ import org.apache.tika.parser.AutoDetectParser;
 import org.apache.tika.parser.ParseContext;
 import org.apache.tika.parser.Parser;
 import org.apache.tika.parser.RecursiveParserWrapper;
+import org.apache.tika.sax.AbstractRecursiveParserWrapperHandler;
 import org.apache.tika.sax.BodyContentHandler;
 import org.junit.Before;
 import org.junit.Test;
@@ -42,10 +43,6 @@ public class MboxParserTest extends TikaTest {
     private TypeDetector typeDetector;
     private MboxParser mboxParser;
 
-    private static InputStream getStream(String name) {
-        return MboxParserTest.class.getClass().getResourceAsStream(name);
-    }
-
     @Before
     public void setUp() throws Exception {
         typeDetector = new TypeDetector();
@@ -62,7 +59,7 @@ public class MboxParserTest extends TikaTest {
         ContentHandler handler = new BodyContentHandler();
         Metadata metadata = new Metadata();
 
-        try (InputStream stream = getStream("/test-documents/simple.mbox")) {
+        try (InputStream stream = getResourceAsStream("/test-documents/simple.mbox")) {
             mboxParser.parse(stream, handler, metadata, recursingContext);
         }
 
@@ -88,7 +85,7 @@ public class MboxParserTest extends TikaTest {
         ContentHandler handler = new BodyContentHandler();
         Metadata metadata = new Metadata();
 
-        try (InputStream stream = getStream("/test-documents/headers.mbox")) {
+        try (InputStream stream = getResourceAsStream("/test-documents/headers.mbox")) {
             mboxParser.parse(stream, handler, metadata, recursingContext);
         }
 
@@ -111,7 +108,7 @@ public class MboxParserTest extends TikaTest {
         ContentHandler handler = new BodyContentHandler();
         Metadata metadata = new Metadata();
 
-        try (InputStream stream = getStream("/test-documents/multiline.mbox")) {
+        try (InputStream stream = getResourceAsStream("/test-documents/multiline.mbox")) {
             mboxParser.parse(stream, handler, metadata, recursingContext);
         }
 
@@ -126,7 +123,7 @@ public class MboxParserTest extends TikaTest {
         ContentHandler handler = new BodyContentHandler();
         Metadata metadata = new Metadata();
 
-        try (InputStream stream = getStream("/test-documents/quoted.mbox")) {
+        try (InputStream stream = getResourceAsStream("/test-documents/quoted.mbox")) {
             mboxParser.parse(stream, handler, metadata, recursingContext);
         }
 
@@ -139,7 +136,7 @@ public class MboxParserTest extends TikaTest {
         ContentHandler handler = new BodyContentHandler();
         Metadata metadata = new Metadata();
 
-        try (InputStream stream = getStream("/test-documents/complex.mbox")) {
+        try (InputStream stream = getResourceAsStream("/test-documents/complex.mbox")) {
             mboxParser.parse(stream, handler, metadata, recursingContext);
         }
 
@@ -162,7 +159,7 @@ public class MboxParserTest extends TikaTest {
         ParseContext context = new ParseContext();
         context.set(Parser.class, new AutoDetectParser());
 
-        try (InputStream stream = getStream("/test-documents/single_mail.mbox")) {
+        try (InputStream stream = getResourceAsStream("/test-documents/single_mail.mbox")) {
             mboxParser.parse(stream, handler, metadata, context);
         }
         
@@ -176,7 +173,7 @@ public class MboxParserTest extends TikaTest {
         assertEquals(2, metadataList.size());
         assertEquals("application/mbox", metadataList.get(0).get(Metadata.CONTENT_TYPE));
         assertEquals("message/rfc822", metadataList.get(1).get(Metadata.CONTENT_TYPE));
-        assertContains("body 2", metadataList.get(1).get(RecursiveParserWrapper.TIKA_CONTENT));
-        assertNotContained("body 1", metadataList.get(1).get(RecursiveParserWrapper.TIKA_CONTENT));
+        assertContains("body 2", metadataList.get(1).get(AbstractRecursiveParserWrapperHandler.TIKA_CONTENT));
+        assertNotContained("body 1", metadataList.get(1).get(AbstractRecursiveParserWrapperHandler.TIKA_CONTENT));
     }
 }
