@@ -17,6 +17,9 @@
 
 package org.apache.tika.server;
 
+import java.io.ByteArrayInputStream;
+import java.io.InputStream;
+import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashSet;
@@ -62,6 +65,12 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 public class TikaServerCli {
+
+
+    //used in spawn-child mode
+    private static final long DEFAULT_MAX_FILES = 100000;
+
+
     public static final int DEFAULT_PORT = 9998;
     private static final int DEFAULT_DIGEST_MARK_LIMIT = 20*1024*1024;
     public static final String DEFAULT_HOST = "localhost";
@@ -75,6 +84,11 @@ public class TikaServerCli {
             "drive or a webpage from your intranet.  See CVE-2015-3271.\n"+
             "Please make sure you know what you are doing.";
 
+    private static final List<String> ONLY_IN_SPAWN_CHILD_MODE =
+            Arrays.asList(new String[] { "taskTimeoutMillis", "taskPulseMillis",
+            "pingTimeoutMillis", "pingPulseMillis", "maxFiles", "javaHome", "maxRestarts",
+            "childStatusFile", "maxChildStartupMillis", "tmpFilePrefix"});
+
     private static Options getOptions() {
         Options options = new Options();
         options.addOption("C", "cors", true, "origin allowed to make CORS requests (default=NONE)\nall allowed if \"all\"");
@@ -88,23 +102,76 @@ public class TikaServerCli {
         options.addOption("?", "help", false, "this help message");
         options.addOption("enableUnsecureFeatures", false, "this is required to enable fileUrl.");
         options.addOption("enableFileUrl", false, "allows user to pass in fileUrl instead of InputStream.");
+        options.addOption("spawnChild", false, "whether or not to spawn a child process for robustness");
+        options.addOption("taskTimeoutMillis", true, "Only in spawn child mode: how long to wait for a task (e.g. parse) to finish");
+        options.addOption("taskPulseMillis", true, "Only in spawn child mode: how often to check if a task has timed out.");
+        options.addOption("pingTimeoutMillis", true, "Only in spawn child mode: how long to wait to wait for a ping and/or ping response.");
+        options.addOption("pingPulseMillis", true, "Only in spawn child mode: how often to check if a ping has timed out.");
+        options.addOption("maxChildStartupMillis", true, "Only in spawn child mode: Maximum number of millis to wait for the child process to startup.");
+        options.addOption("maxRestarts", true, "Only in spawn child mode: how many times to restart child process, default is -1 (always restart)");
+        options.addOption("maxFiles", true, "Only in spawn child mode: shutdown server after this many files (to handle parsers that might introduce " +
+                "slowly building memory leaks); the default is "+DEFAULT_MAX_FILES +". Set to -1 to turn this off.");
+        options.addOption("javaHome", true, "Only in spawn child mode: override system property JAVA_HOME for calling java for the child process");
+        options.addOption("child", false, "Only in spawn child mode: this process is a child process -- do not use this! " +
+                "Should only be invoked by parent process");
+        options.addOption("childStatusFile", true, "Only in spawn child mode: temporary file used as mmap to communicate " +
+                "with parent process -- do not use this! Should only be invoked by parent process.");
+        options.addOption("tmpFilePrefix", true, "Only in spawn child mode: prefix for temp file - for debugging only");
 
         return options;
     }
 
     public static void main(String[] args) {
         LOG.info("Starting {} server", new Tika());
-
         try {
-            Options options = getOptions();
+            execute(args);
+        } catch (Exception e) {
+            e.printStackTrace();
+            LOG.error("Can't start: ", e);
+            System.exit(-1);
+        }
+    }
 
-            CommandLineParser cliParser = new GnuParser();
-            CommandLine line = cliParser.parse(options, args);
+    private static void execute(String[] args) throws Exception {
+        Options options = getOptions();
 
+        CommandLineParser cliParser = new GnuParser();
+
+        //need to strip out -J (child jvm opts) from this parse
+        //they'll be processed correctly in args in the watch dog
+        //and they won't be needed in legacy.
+        CommandLine line = cliParser.parse(options, stripChildArgs(args));
+        if (line.hasOption("spawnChild")) {
+            TikaServerWatchDog watchDog = new TikaServerWatchDog();
+            watchDog.execute(args, configureServerTimeouts(line));
+        } else {
+            if (! line.hasOption("child")) {
+                //make sure the user didn't misunderstand the options
+                for (String childOnly : ONLY_IN_SPAWN_CHILD_MODE) {
+                    if (line.hasOption(childOnly)) {
+                        System.err.println("The option '" + childOnly +
+                                "' can only be used with '-spawnChild'");
+                        usage(options);
+                    }
+                }
+            }
+            executeLegacy(line, options);
+        }
+    }
+
+    private static String[] stripChildArgs(String[] args) {
+        List<String> ret = new ArrayList<>();
+        for (int i = 0; i < args.length; i++) {
+            if (! args[i].startsWith("-J")) {
+                ret.add(args[i]);
+            }
+        }
+        return ret.toArray(new String[ret.size()]);
+    }
+
+    private static void executeLegacy(CommandLine line, Options options) throws Exception {
             if (line.hasOption("help")) {
-                HelpFormatter helpFormatter = new HelpFormatter();
-                helpFormatter.printHelp("tikaserver", options);
-                System.exit(-1);
+                usage(options);
             }
 
             String host = DEFAULT_HOST;
@@ -196,15 +263,39 @@ public class TikaServerCli {
                 inputStreamFactory = new DefaultInputStreamFactory();
             }
 
-            TikaResource.init(tika, digester, inputStreamFactory);
+            ServerStatus serverStatus;
+            //if this is a child process
+            if (line.hasOption("child")) {
+                serverStatus = new ServerStatus();
+                //redirect!!!
+                InputStream in = System.in;
+                System.setIn(new ByteArrayInputStream(new byte[0]));
+                System.setOut(System.err);
+
+                long maxFiles = DEFAULT_MAX_FILES;
+                if (line.hasOption("maxFiles")) {
+                    maxFiles = Long.parseLong(line.getOptionValue("maxFiles"));
+                }
+
+                ServerTimeouts serverTimeouts = configureServerTimeouts(line);
+                String childStatusFile = line.getOptionValue("childStatusFile");
+                Thread serverThread =
+                new Thread(new ServerStatusWatcher(serverStatus, in,
+                        Paths.get(childStatusFile), maxFiles, serverTimeouts));
+
+                serverThread.start();
+            } else {
+                serverStatus = new ServerStatus(true);
+            }
+            TikaResource.init(tika, digester, inputStreamFactory, serverStatus);
             JAXRSServerFactoryBean sf = new JAXRSServerFactoryBean();
 
             List<ResourceProvider> rCoreProviders = new ArrayList<>();
             rCoreProviders.add(new SingletonResourceProvider(new MetadataResource()));
             rCoreProviders.add(new SingletonResourceProvider(new RecursiveMetadataResource()));
-            rCoreProviders.add(new SingletonResourceProvider(new DetectorResource()));
+            rCoreProviders.add(new SingletonResourceProvider(new DetectorResource(serverStatus)));
             rCoreProviders.add(new SingletonResourceProvider(new LanguageResource()));
-            rCoreProviders.add(new SingletonResourceProvider(new TranslateResource()));
+            rCoreProviders.add(new SingletonResourceProvider(new TranslateResource(serverStatus)));
             rCoreProviders.add(new SingletonResourceProvider(new TikaResource()));
             rCoreProviders.add(new SingletonResourceProvider(new UnpackerResource()));
             rCoreProviders.add(new SingletonResourceProvider(new TikaMimeTypes()));
@@ -241,9 +332,48 @@ public class TikaServerCli {
             manager.registerBindingFactory(JAXRSBindingFactory.JAXRS_BINDING_ID, factory);
             sf.create();
             LOG.info("Started Apache Tika server at {}", url);
-        } catch (Exception ex) {
-            LOG.error("Can't start", ex);
-            System.exit(-1);
-        }
     }
+
+    private static void usage(Options options) {
+        HelpFormatter helpFormatter = new HelpFormatter();
+        helpFormatter.printHelp("tikaserver", options);
+        System.exit(-1);
+    }
+
+    private static ServerTimeouts configureServerTimeouts(CommandLine line) {
+        ServerTimeouts serverTimeouts = new ServerTimeouts();
+        /*TODO -- add these in
+        if (line.hasOption("childProcessStartupMillis")) {
+            serverTimeouts.setChildProcessStartupMillis(
+                    Long.parseLong(line.getOptionValue("childProcessStartupMillis")));
+        }
+        if (line.hasOption("childProcessShutdownMillis")) {
+            serverTimeouts.setChildProcessShutdownMillis(
+                    Long.parseLong(line.getOptionValue("childProcesShutdownMillis")));
+        }*/
+        if (line.hasOption("taskTimeoutMillis")) {
+            serverTimeouts.setTaskTimeoutMillis(
+                    Long.parseLong(line.getOptionValue("taskTimeoutMillis")));
+        }
+        if (line.hasOption("pingTimeoutMillis")) {
+            serverTimeouts.setPingTimeoutMillis(
+                    Long.parseLong(line.getOptionValue("pingTimeoutMillis")));
+        }
+        if (line.hasOption("pingPulseMillis")) {
+            serverTimeouts.setPingPulseMillis(
+                    Long.parseLong(line.getOptionValue("pingPulseMillis")));
+        }
+
+        if (line.hasOption("maxRestarts")) {
+            serverTimeouts.setMaxRestarts(Integer.parseInt(line.getOptionValue("maxRestarts")));
+        }
+
+        if (line.hasOption("maxChildStartupMillis")) {
+            serverTimeouts.setMaxChildStartupMillis(
+                    Long.parseLong(line.getOptionValue("maxChildStartupMillis")));
+        }
+
+        return serverTimeouts;
+    }
+
 }
