@@ -33,6 +33,7 @@ import org.apache.commons.compress.archivers.tar.TarArchiveInputStream;
 import org.apache.cxf.jaxrs.JAXRSServerFactoryBean;
 import org.apache.cxf.jaxrs.client.WebClient;
 import org.apache.cxf.jaxrs.lifecycle.SingletonResourceProvider;
+import org.apache.tika.server.resource.TikaResource;
 import org.apache.tika.server.writer.TarWriter;
 import org.apache.tika.server.resource.UnpackerResource;
 import org.apache.tika.server.writer.ZipWriter;
@@ -196,4 +197,14 @@ public class UnpackerResourceTest extends CXFTestBase {
         assertTrue(responseMsg.contains("test"));
     }
 
+    @Test
+    public void testPDFImages() throws Exception {
+        Response response = WebClient.create(endPoint + UNPACKER_PATH)
+                .header(TikaResource.X_TIKA_PDF_HEADER_PREFIX+"ExtractInlineImages", "true")
+                .accept("application/zip")
+                .put(ClassLoader.getSystemResourceAsStream("testOCR.pdf"));
+        Map<String, String> results = readZipArchive((InputStream)response.getEntity());
+        assertTrue(results.containsKey("image0.png"));
+        assertEquals("7c2f14acbb737672a1245f4ceb50622a", results.get("image0.png"));
+    }
 }
